(function() {
    "use strict";

    var yoyakuDAO = {};

    yoyakuDAO.CONST_OK = 0;
    yoyakuDAO.CONST_NG = 1;

    //レコード一覧画面のインライン編集またはレコード編集画面が表示された時
    kintone.events.on(['app.record.index.edit.show', 'app.record.edit.show'], function(event) {
        //編集前の店舗、予約日付、予約時刻、担当者を保存しておく
        var record = event.record;
        yoyakuDAO.frmShop = record['店舗']['value'];
        yoyakuDAO.frmDate = record['予約日付']['value'];
        yoyakuDAO.frmTime = record['予約時刻']['value'];
        yoyakuDAO.frmPerson = record['担当者']['value'];

        if (record['予約不可']['value'][0] === "不可" || record['キャンセル']['value'][0] === "C") {
            //予約不可またはキャンセル済の場合は入力不可
            record['お客様番号']['disabled'] = true;
            record['お名前カナ']['disabled'] = true;
            record['お名前漢字']['disabled'] = true;
            record['電話番号']['disabled'] = true;
        }
        return event;
    });

    //各フィールドの値が変更された時
    var eventName = ['app.record.index.edit.change.予約不可',
        'app.record.index.edit.change.キャンセル',
        'app.record.edit.change.予約不可',
        'app.record.edit.change.キャンセル'];
    kintone.events.on(eventName, function(event) {
        var record = event.record;
        if (record['予約不可']['value'][0] === "不可") {
            record['キャンセル']['value'] = ["C"];
        }
        if (record['キャンセル']['value'][0] === "C") {
            //予約不可またはキャンセルの場合は入力不可
            record['お客様番号']['value'] = "";
            record['お客様番号']['disabled'] = true;
            record['お名前カナ']['value'] = "";
            record['お名前カナ']['disabled'] = true;
            record['お名前漢字']['value'] = "";
            record['お名前漢字']['disabled'] = true;
            record['電話番号']['value'] = "";
            record['電話番号']['disabled'] = true;
        } else {
            record['お客様番号']['disabled'] = false;
            record['お名前カナ']['disabled'] = false;
            record['お名前漢字']['disabled'] = false;
            record['電話番号']['disabled'] = false;
        }
        return event;
    });

    //保存ボタンが押された時
    kintone.events.on(['app.record.index.edit.submit', 'app.record.edit.submit'], function(event) {
        var record = event.record;

        //現在の店舗、予約日付、予約時刻、担当者を取り出す
        var newShop = record['店舗']['value'];
        var newDate = record['予約日付']['value'];
        var newTime = record['予約時刻']['value'];
        var newPerson = record['担当者']['value'];

        if (newShop === yoyakuDAO.frmShop && newDate === yoyakuDAO.frmDate && newTime === yoyakuDAO.frmTime && newPerson === yoyakuDAO.frmPerson) {
            //店舗、予約日付、予約時刻、担当者に変更無し
            return event;
        }
        //店舗、予約日付、予約時刻、担当者の1つ以上が変更された場合
        if (window.confirm('予約の店舗・日時を変更しますか？')) {
            //変更希望の店舗・日時・担当者が空いているかを検索する
            var appId = kintone.app.getId();
            var queryCondition = '店舗 = "' + newShop + '" and 予約日付 = "' +
            newDate + '" and 予約時刻 = "' + newTime + '" and 担当者 = "' +
            newPerson + '" and 予約不可 not in ("不可") and お名前カナ = ""';

            var params = {
                "app": appId,
                "query": queryCondition
            };
            return kintone.api(kintone.api.url('/k/v1/records', true), 'GET', params).then(function(resp) {
                // 検索成功時
                if (resp.records.length === 0) {
                    // 検索結果が空
                    event.error = "入力された店舗・日時・担当者が見つかりません。";
                    // 4つのフィールドを編集前に戻してkintoneに戻る
                    record['店舗']['value'] = yoyakuDAO.frmShop;
                    record['予約日付']['value'] = yoyakuDAO.frmDate;
                    record['予約時刻']['value'] = yoyakuDAO.frmTime;
                    record['担当者']['value'] = yoyakuDAO.frmPerson;
                    return event;
                }
                // 検索結果が空ではない
                // 店舗・予約日付・予約時刻・担当者を入れ替えて更新
                var updrecord = resp.records[0];
                var updparams = {
                    "app": appId,
                    "id": updrecord['$id']['value'],
                    "revision": updrecord['$revision']['value'],
                    "record": {
                        "店舗": {"value": yoyakuDAO.frmShop},
                        "予約日付": {"value": yoyakuDAO.frmDate},
                        "予約時刻": {"value": yoyakuDAO.frmTime},
                        "担当者": {"value": yoyakuDAO.frmPerson},
                        "キャンセル": {"value": ["C"]}
                    }
                };
                return kintone.api(kintone.api.url('/k/v1/record', true), 'PUT', updparams).then(function(updresp) {
                    // 更新成功時
                    alert("変更を完了しました。");
                    return event;
                }, function(updresp) {
                    // 更新エラー時
                    if (updresp.message !== undefined) {
                        console.log(updresp.message);
                    }
                    event.error = "予約の変更に失敗しました。";
                    // 4つのフィールドを編集前に戻してkintoneに戻る
                    record['店舗']['value'] = yoyakuDAO.frmShop;
                    record['予約日付']['value'] = yoyakuDAO.frmDate;
                    record['予約時刻']['value'] = yoyakuDAO.frmTime;
                    record['担当者']['value'] = yoyakuDAO.frmPerson;
                    return event;
                });
            }, function(resp) {
                // 検索エラー時
                event.error = "入力された店舗・日時・担当者に変更できません。";
                // 4つのフィールドを編集前に戻してkintoneに戻る
                record['店舗']['value'] = yoyakuDAO.frmShop;
                record['予約日付']['value'] = yoyakuDAO.frmDate;
                record['予約時刻']['value'] = yoyakuDAO.frmTime;
                record['担当者']['value'] = yoyakuDAO.frmPerson;
                if (resp.message !== undefined) {
                    console.log(resp.message);
                }
                return event;
            });
        }
    });
})();

